@@ -21,7 +21,7 @@ from django.db import models, connections
 import psycopg2
 import sqlparse
 
-from djaq.query import DjangoQuery as DQ
+from djaq import DjaqQuery as DQ
 from djaq.app_utils import (
     get_schema,
     get_model_classes,
@@ -29,6 +29,7 @@ from djaq.app_utils import (
     get_model_details,
     get_whitelist,
 )
+import ipdb
 
 logger = logging.getLogger(__name__)
 
@@ -40,26 +41,47 @@ def wl():
     return whitelist
 
 
+def get_params(data):
+    model = data.get("model")
+    output = data.get("output")
+    where = data.get("where")
+    order_by = data.get("order_by")
+    return model, output, where, order_by
+
+
 @login_required
 @csrf_exempt
 def query_view(request):
 
     if request.method == "POST":
+        ipdb.set_trace()
 
-        q = request.POST.get("query")
+        model, output, where, order_by = get_params(request.POST.get("queries")[0])
         limit = request.POST.get("limit", 10)
         offset = request.POST.get("offset", 0)
         sql = request.POST.get("sql", False) == "true"
         try:
             if sql:
-                s = DQ(q, whitelist=wl()).offset(offset).limit(limit).parse()
+                s = (
+                    DQ(model, output, whitelist=wl())
+                    .where(where)
+                    .order_by(order_by)
+                    .offset(offset)
+                    .limit(limit)
+                    .sql()
+                )
                 s = sqlparse.format(s, reindent=True, keyword_case="upper")
                 r = {"result": s}
             else:
                 try:
                     r = {
                         "result": list(
-                            DQ(q, whitelist=wl()).offset(offset).limit(limit).dicts()
+                            DQ(model, output, whitelist=wl())
+                            .where(where)
+                            .order_by(order_by)
+                            .offset(offset)
+                            .limit(limit)
+                            .dicts()
                         )
                     }
                 except Exception as e:
@@ -82,10 +104,13 @@ def query_view(request):
 @login_required
 @csrf_exempt
 def sql_view(request):
-
-    q = request.POST.get("query")
+    # ipdb.set_trace()
+    print(request.body)
+    request_data = json.loads(request.body.decode("utf-8"))
+    print(request_data)
+    model, output, where, order_by = get_params(request_data.get("queries")[0])
     try:
-        s = DQ(q).parse()
+        s = DQ(model, output).where(where).order_by(order_by).sql()
         s = sqlparse.format(s, reindent=True, keyword_case="upper")
         r = {"result": s}
         return JsonResponse(r)
